import hashlib
import os
from datetime import datetime

import cv2
from easyocr import easyocr


class Vagon_number_recognize_class:
    def __init__(self):
        self.reader = easyocr.Reader(['ru'], gpu= False)

    def work_image_cut(self, filepath):
        bounds = self.reader.readtext(filepath, allowlist='0123456789')
        str_res = ''
        index = 0
        sum_conf = 0.0
        count_conf = 0
        avg_conf = 0.0
        if (len(bounds) > 0):
            for bound in bounds:
                if bound[2] > 0.5:
                    str_res += bound[1]
                    sum_conf += bound[2]
                    count_conf += 1
                    avg_conf = sum_conf / count_conf
        if len(str_res) > 8:
            len_str_res = len(str_res)
            min_index = 0
            min_conf = 1.1
            index = 0
            for bound in bounds:
                if (bound[2] < min_conf) and (bound[2] > 0.5):
                    min_index = index
                    min_conf = bound[2]
                index += 1
            if (len_str_res - len(bounds[min_index][1])) >= 8:
                str_res = ''
                index = 0
                sum_conf = 0.0
                count_conf = 0
                avg_conf = 0.0
                for bound in bounds:
                    if (bound[2] > 0.5) and index != min_index:
                        str_res += bound[1]
                        sum_conf += bound[2]
                        count_conf += 1
                        avg_conf = sum_conf / count_conf
                    index += 1
                return (str_res, avg_conf)
            else:
                str_res = ''
                index = 0
                sum_conf = 0.0
                count_conf = 0
                avg_conf = 0.0
                for bound in bounds:
                    if bound[2] > 0.5:
                        if (index != min_index):
                            str_res += bound[1]
                            sum_conf += bound[2]
                            count_conf += 1
                            avg_conf = sum_conf / count_conf
                        else:
                            new_length_without = 8 - (len_str_res - len(bounds[min_index][1]))
                            str_res += bound[1][0:new_length_without]
                            sum_conf += bound[2]
                            count_conf += 1
                            avg_conf = sum_conf / count_conf
                    index += 1
                return (str_res, avg_conf)
        else:
            return (str_res, avg_conf)

    def make_cuts(self, img_vagon, rect_array):
        number_image_cuts = []
        for rect in rect_array:
            number_image_cuts.append(img_vagon[rect.ymin:rect.ymax, rect.xmin:rect.xmax])
        return number_image_cuts

    def work_image(self, img_vagon, rect_array):
        image_text_areas = self.make_cuts(img_vagon, rect_array)
        numbers = []
        for image_text_area in image_text_areas:
            number = {'image_text_area' : image_text_area, 'text' : '', 's' : 0, 'conf' : 0.0}
            h, w = image_text_area.shape[:2]
            number['s'] = h * w
            hash_object = hashlib.md5(bytes(str(datetime.now()), 'utf-8'))
            hash_str = hash_object.hexdigest()
            img_cut_path = 'cut_text_img' + hash_str + '.jpg'
            cv2.imwrite(img_cut_path, image_text_area)
            (text, conf) = self.work_image_cut(img_cut_path)
            number['text'] = text
            number['conf'] = conf
            numbers.append(number)
            os.remove(img_cut_path)
        numbers.sort(key= lambda number : number['s'])
        if len(numbers) == 1:
            return (numbers[0]['text'], numbers[0]['conf'])
        else:
            if (len(numbers[0]['text']) != 8) and (len(numbers[1]['text']) == 8):
                return (numbers[1]['text'], numbers[1]['conf'])
            elif (len(numbers[0]['text']) == 8) and (len(numbers[1]['text']) != 8):
                return (numbers[0]['text'], numbers[0]['conf'])
            else:
                ans = sorted(numbers[0:2], key = lambda number : number['conf'])[0]
                return ans['text'], ans['conf']


import cv2
from tensorflow import keras
import numpy as np

from data.neuro.models import *
from data.result.Class_im import *
import zipfile
import requests
from urllib.parse import urlencode

class Detail_classify_class:
    def __init__(self, detail_classify_model_path):
        base_url = 'https://cloud-api.yandex.net/v1/disk/public/resources/download?'
        public_key = 'https://disk.yandex.ru/d/k6Wx0bD98fpswQ'  # Сюда вписываете вашу ссылку

        public_key = detail_classify_model_path  # Сюда вписываете вашу ссылку
        # Получаем загрузочную ссылку
        final_url = base_url + urlencode(dict(public_key=public_key))
        response = requests.get(final_url)
        download_url = response.json()['href']

        # Загружаем файл и сохраняем его
        download_response = requests.get(download_url)
        # print(download_response.content)

        # print(download_response.content)
        with open('detail_classify_model.zip', 'wb') as f:
            f.write(download_response.content)

        with zipfile.ZipFile('detail_classify_model.zip', 'r') as zip_ref:
            zip_ref.extractall()
        self.detail_classify_model = keras.models.load_model('detail_classify_model.h5')

    def prepare_img(self, img):
        img_grey = cv2.cvtColor(img, cv2.COLOR_BGR2GRAY)
        img_grey_size = cv2.resize(img_grey, (512, 512))
        data = np.array(img_grey_size, dtype="float") / 255.0
        data = data.reshape((1, 512, 512))
        return data

    def work_img(self, img):
        """work_img(img:openCV frame):Double[0..1] list - classify openCV img with CNN, returns list with double[0..1] values"""
        data = self.prepare_img(img)
        res = self.detail_classify_model.predict(data)
        res_list = res[0].tolist()
        return res_list

    def classify(self, image_initial):
        """classify(img:openCV frame):Class_im - classify openCV img with CNN, returns Class_im"""
        res_list = self.work_img(image_initial)
        res_index = res_list.index(max(res_list))
        class_detail = Class_detail(res_index)
        conf = res_list[res_index]
        return Class_detail_result(class_detail, round(conf, 2))


from enum import Enum
class Class_detail(Enum):
    rama = 0
    vagon = 1

class Class_detail_result:
    def __init__(self, class_detail, conf):
        self.class_detail = class_detail
        self.conf = conf


from enum import Enum


class Rama_Prod(Enum):
    bejickaya = 0
    ruzhimmash = 1
    no_rama = 2

class Rama_Prod_Conf:

    def __init__(self, rama_prod, conf):
        self.rama_prod = rama_prod
        self.conf = conf


import json
import os

import cv2
import torch
import zipfile
import requests
from urllib.parse import urlencode
from data.result.Rect import Rect

"""module for detecting number in vagon"""
class Vagon_number_detect_class:
    """module for detecting number in vagon"""

    # reads yolov5 taught model from yandex-disk and includes it in class example
    def __init__(self, model_path, yolo_path):
        """reads yolov5 taught model from yandex-disk and includes it in class example"""
        base_url = 'https://cloud-api.yandex.net/v1/disk/public/resources/download?'
        public_key = model_path  # Сюда вписываете вашу ссылку
        # Получаем загрузочную ссылку
        final_url = base_url + urlencode(dict(public_key=public_key))
        response = requests.get(final_url)
        download_url = response.json()['href']
        # Загружаем файл и сохраняем его
        download_response = requests.get(download_url)
        zip_path = 'vagon_number_detect.zip'
        # print(download_response.content)
        with open(zip_path, 'wb') as f:
            f.write(download_response.content)

        with zipfile.ZipFile(zip_path, 'r') as zip_ref:
            zip_ref.extractall()
        weights_file_path = 'vagon_number_detect.pt'
        self.model = torch.hub.load(yolo_path, 'custom', weights_file_path, source='local')



    #получить JSON с результатами yolo
    def work_img(self, img_path, size):
        """get JSON with yolo_results from img from img_path"""
        results = self.model([img_path], size=size)
        json_res = results.pandas().xyxy[0].to_json(orient="records")
        res2 = json.loads(json_res)
        return res2

    # получить координаты прямоугольников с номерами
    def vagon_rama_detect(self, img_path, size):
        """get Rect object with rama coordinates from img from img_path"""
        json_res = self.work_img(img_path, size)
        number_rects = []
        for dict_text_area in json_res:
            if dict_text_area['confidence']>0.25:
                number_rect = Rect(   xmin=int(float(dict_text_area['xmin'])),
                                        xmax=int(float(dict_text_area['xmax'])),
                                        ymin=int(float(dict_text_area['ymin'])),
                                        ymax=int(float(dict_text_area['ymax']))
                                )
                number_rects.append(number_rect)
        return number_rects





from data.result.word_compare_result import Word_compare_result


class Rect:
    # прочитать из json результата йоло

    @staticmethod
    def get_rect_from_yolo_json(yolo_json):
        xmin = int(float(yolo_json['xmin']))
        xmax = int(float(yolo_json['xmax']))
        ymin = int(float(yolo_json['ymin']))
        ymax = int(float(yolo_json['ymax']))
        rect = Rect(xmin, xmax, ymin, ymax)
        return rect

    def __init__(self, xmin=0, xmax=0, ymin=0, ymax=0):
        self.xmin = xmin
        self.xmax = xmax
        self.ymin = ymin
        self.ymax = ymax

    # Найти IOU между этим прямоугольником и другим, данным в объекте
    def IoU(self, rect):
        def IOU(xmin, xmax, ymin, ymax, xmin_t, xmax_t, ymin_t, ymax_t):
            I = 0
            U = 0
            xmin_U = min(xmin, xmin_t)
            xmax_U = max(xmax, xmax_t)
            ymin_U = min(ymin, ymin_t)
            ymax_U = max(ymax, ymax_t)
            h = ymax_U - ymin_U
            w = xmax_U - xmin_U
            for i in range(xmin_U, xmax_U):
                for j in range(ymin_U, ymax_U):
                    flag = ((i <= xmax) and (i >= xmin) and (j <= ymax) and (j >= ymin))
                    flag_t = ((i <= xmax_t) and (i >= xmin_t) and (j <= ymax_t) and (j >= ymin_t))
                    if (flag and flag_t):
                        I += 1
                    if (flag or flag_t):
                        U += 1
            resultat = I / float(U)
            return resultat
        return IOU(self.xmin, self.xmax, self.ymin, self.ymax,
                   rect.xmin, rect.xmax, rect.ymin, rect.ymax)

    def __str__(self):
        res = ('xmin = ' + str(self.xmin) + ', xmax = ' + str(self.xmax) + ', ymin = ' + str(self.ymin) +
               ', ymax = ' + str(self.ymax))
        return res

    def intersection(self, rect):
        h = max(self.ymax, rect.ymax) - min(self.ymin, rect.ymin)
        w = max(self.xmax, rect.xmax) - min(self.xmin, rect.xmin)
        I = 0
        U = 0
        for i in range(0, w):
            for j in range(0, h):
                flag = ((i <= self.xmax) and (i >= self.xmin) and (j <= self.ymax) and (j >= self.ymin))
                flag_t = ((i <= rect.xmax) and (i >= rect.xmin) and (j <= rect.ymax) and (j >= rect.ymin))
                if (flag and flag_t):
                    I += 1
        return I

    def union(self, rect):
        new_xmin = min(self.xmin, rect.xmin)
        new_ymin = min(self.ymin, rect.ymin)
        new_xmax = max(self.xmax, rect.xmax)
        new_ymax = max(self.ymax, rect.ymax)
        return Rect(new_xmin, new_xmax, new_ymin, new_ymax)

    def sort_rects_for_w_min(self, rect):
        if (self.xmax - self.xmin) < (rect.xmax - rect.xmin):
            return self, rect
        else:
            return rect, self

    def equal_x_for_percent(self, rect, percent):
        min_rect, max_rect = self.sort_rects_for_w_min(rect)
        x_intersect = min(max_rect.xmax - min_rect.xmin, min_rect.xmax - max_rect.xmin)
        if x_intersect < 0:
            x_intersect = 0
        wmin = min_rect.xmax - min_rect.xmin
        cur_percent = x_intersect / float(wmin)
        return (cur_percent > percent)

class Yolo_label_Rect:

    @staticmethod
    def build_from_2D_array(data, h, w):
        return Yolo_label_Rect(data[1], data[3], data[2], data[4], h, w)

    def __init__(self, xc=0.0, ow=0.0, yc=0.0, oh=0.0, h=0.0, w=0.0):
        self.xc = xc
        self.ow = ow
        self.yc = yc
        self.oh = oh
        self.w = w
        self.h = h


    def build_rect(self):
        xmin_t = int((self.xc - self.ow / 2) * self.w)
        xmax_t = int((self.xc + self.ow / 2) * self.w)
        ymin_t = int((self.yc - self.oh / 2) * self.h)
        ymax_t = int((self.yc + self.oh / 2) * self.h)
        return Rect(xmin_t, xmax_t, ymin_t, ymax_t)


class Letter_In_Image:
    def __init__(self, letter, rect = None, confidence=0.0):
        self.letter = letter
        self.rect = rect
        self.confidence = float(confidence)


    @staticmethod
    def get_letter_in_image_from_yolo_json(letter_json):
        return Letter_In_Image(letter_json['name'], Rect.get_rect_from_yolo_json(letter_json), letter_json['confidence'])

    def __eq__(self, other):
        return (other is Letter_In_Image) & (self.letter == other.letter)

    def __hash__(self):
        return hash(self.letter)

    def __str__(self):
        res_dict = {'letter' : self.letter, 'xmin' : self.rect.xmin, 'xmax' : self.rect.xmax, 'ymin' : self.rect.ymin, 'ymax' : self.rect.ymax, 'confidence' : self.confidence}
        return str(res_dict)


class Letters_In_Image:
    def __init__(self):
        self.letters = []



    @staticmethod
    def compare(letters1, letters2):
        if len(letters1.letters) == 0:
            return Word_compare_result(0)
        else:
            let1 = letters1.make_word()
            let2 = letters2.make_word()
            if let1 == let2:
                return Word_compare_result(3)
            count = 0
            for l in let1:
                if let2.find(l) > -1:
                    count += 1
            per_cent = count / float(len(let2))
            return Word_compare_result(1) if per_cent < 0.5 else Word_compare_result(2)


    @staticmethod
    def get_letters_in_image_from_yolo_json(letters_json):
        letters_in_image = Letters_In_Image()
        for letter_json in letters_json:
            letters_in_image.letters.append(Letter_In_Image.get_letter_in_image_from_yolo_json(letter_json))
        return letters_in_image

    def sort_letters(self):
        self.letters.sort(key = lambda letter_in_image: letter_in_image.rect.xmin)

    def delete_intersections(self):
        new_letters = []
        i = 0
        while i < len(self.letters) - 1:
            IoU = self.letters[i].rect.IoU(self.letters[i+1].rect)
            if IoU > 0.5:
                new_letter = self.letters[i] if self.letters[i].confidence > self.letters[i + 1].confidence else self.letters[i + 1]
                i += 2
            else:
                new_letter = self.letters[i]
                i += 1
            new_letters.append(new_letter)
        if (i == len(self.letters) - 1):
            new_letters.append(self.letters[i])
        self.letters = new_letters

    def find_x_intersection(self):
        otvet_index_1 = 0
        otvet_index_2 = 0
        index1 = 0
        flag = False
        while not(flag) and (index1 < len(self.letters) - 1):
            index2 = index1 + 1
            while not(flag) and (index2 < len(self.letters)):
                if self.letters[index1].rect.equal_x_for_percent(self.letters[index2].rect, 0.6):
                    flag = True
                    otvet_index_1 = index1
                    otvet_index_2 = index2
                index2 += 1
            index1 += 1
        return (flag, otvet_index_1, otvet_index_2)

    def delete_x_intersections(self):
        has_intersections_flag = self.has_intersections()
        while has_intersections_flag:
            self.delete_x_intersections_iter()
            has_intersections_flag = self.has_intersections()

    def has_intersections(self):
        index1 = 0
        flag = False
        while not (flag) and (index1 < len(self.letters) - 1):
            index2 = index1 + 1
            while not (flag) and (index2 < len(self.letters)):
                if self.letters[index1].rect.IoU(self.letters[index2].rect) > 0.5:
                    flag = True
                index2 += 1
            index1 += 1
        return flag



    def delete_x_intersections_iter(self):
        (flag, index1, index2) = self.find_x_intersection()
        while flag:
            if flag:
                if (self.letters[index1].rect.xmax - self.letters[index1].rect.xmin) < (self.letters[index2].rect.xmax - self.letters[index2].rect.xmin):
                    self.letters.remove(self.letters[index1])
                else:
                    self.letters.remove(self.letters[index2])
            (flag, index1, index2) = self.find_x_intersection()

    def make_word(self):
        if (len(self.letters) == 0):
            return '_'
        else:
            res = ''
            for letter in self.letters:
                res += letter.letter
            return res

    def get_avg_conf(self):
        sum_conf = 0.0
        count_conf = 0
        for letter in self.letters:
            sum_conf = sum_conf + letter.confidence
            count_conf += 1
        if count_conf==0:
            res = 0.0
        else:
            res = sum_conf / count_conf
        return res

    def __str__(self):
        res = ''
        for letter_in_image in self.letters:
            res = res + letter_in_image.__str__() + '\n'
        return res




# import os
#
# import cv2
#
# from danila.danila_v1 import Danila_v1
# from danila.danila_v2 import Danila_v2
# from danila.danila_v3 import Danila_v3
from danila.danila_v4 import Danila_v4

"""main module for user"""


class Danila:
    """main class for user"""
    def __init__(self, version, yolov5_dir):
        if (version == 4):
            self.danila = Danila_v4(yolov5_dir)
        else:
            self.danila = Danila_v4(yolov5_dir)
    # returns string - class of rama using CNN network
    # img - openCV frame

    def detail_classify(self, img, size = 256):
        return self.danila.detail_classify(img, size)

    def detail_text_detect(self, img, size = 256):
        return self.danila.detail_text_detect(img, size)

    def detail_text_recognize(self, img, size = 256):
        return self.danila.detail_text_recognize(img, size)

    def rama_classify(self, img, size = 256):
        """rama_classify(Img : openCv frame): String - returns class of rama using CNN network"""
        """rama_classify uses Rama_classify_class method - classify(Img)"""
        return self.danila.rama_classify(img, size)

    # returns openCV frame with rama from openCV frame\
    # def rama_detect(self, img, size = 256):
    #     """rama_detect(img : openCV img) -> openCV image with drawn rama rectangle"""
    #     return self.danila.rama_detect(img, size)
    #
    # # returns openCV image with cut_rama
    # def rama_cut(self, img, size = 256):
    #     """rama_cut(img : openCV img) -> openCV image of rama rectangle"""
    #     return self.danila.rama_cut(img, size)

    #
    # returns openCV cut rama with drawn text areas
    def rama_text_detect_cut(self, img, size = 256):
        """returns openCV cut rama with drawn text areas"""
        return self.danila.rama_text_detect_cut(img, size)

    # returns openCV img with drawn text areas
    # def rama_text_detect(self, img, size = 256):
    #     """returns openCV img with drawn text areas"""
    #     return self.danila.text_detect(img, size)
    # returns dict {'number', 'prod', 'year'} for openCV rama img or 'no_rama'
    def rama_text_recognize(self, img, size = 256):
        """returns dict {'number', 'prod', 'year'} for openCV rama img or 'no_rama'"""
        return self.danila.rama_text_recognize(img, size)

    # returns openCV img with drawn number areas
    def vagon_number_detect(self, img, size = 256):
        """returns openCV img with drawn number areas"""
        return self.danila.vagon_number_detect(img, size)

    def vagon_number_recognize(self, img, size = 256):
        return self.danila.vagon_number_recognize(img,size)


RAMA_CLASSIFY_MODEL_ADDRESS = 'https://disk.yandex.ru/d/4xtNt0zftHFFVg'
RAMA_BEGICKAYA_DETECT_MODEL_ADDRESS = 'https://disk.yandex.ru/d/sE5iSXYWL3xEkQ'
RAMA_RUZHIMMASH_DETECT_MODEL_ADDRESS = 'https://disk.yandex.ru/d/zAEw0_ALgtBg9Q'
RAMA_RUZHIMMASH_TEXT_DETECT_MODEL_ADDRESS = 'https://disk.yandex.ru/d/rgthm-mMrbhOmw'
RAMA_BEJICKAYA_TEXT_DETECT_MODEL_ADDRESS = 'https://disk.yandex.ru/d/QhicLdm2-PT6oA'
# TEXT_RECOGNIZE_RUZHIMMASH_MODEL_ADDRESS = 'https://disk.yandex.ru/d/C2hDMsbTZMnL8Q' -7500_b7500_r
# TEXT_RECOGNIZE_BEGICKAYA_MODEL_ADDRESS = 'https://disk.yandex.ru/d/C2hDMsbTZMnL8Q'
TEXT_RECOGNIZE_RUZHIMMASH_MODEL_ADDRESS = 'https://disk.yandex.ru/d/GcI-mAtwpb2I-g' # 11000_b_11000_r
TEXT_RECOGNIZE_BEGICKAYA_MODEL_ADDRESS = 'https://disk.yandex.ru/d/GcI-mAtwpb2I-g'  # 11000_b_11000_r

RAMA_PROD_CLASSIFY_MODEL_ADDRESS = 'https://disk.yandex.ru/d/8FI8TaTwaCfbSA'
VAGON_NUMBER_DETECT_MODEL_ADDRESS = 'https://disk.yandex.ru/d/ZGS0sdk1syJmcQ'
DETAIL_CLASSIFY_MODEL_ADDRESS = 'https://disk.yandex.ru/d/Xamk_OYzYAbGhg'
VAGON_NUMBER_RECOGNIZE_MODEL_ADDRESS = 'https://disk.yandex.ru/d/eh714_PrjFyLHg'


import hashlib
import json
import os
import zipfile
from datetime import datetime

import cv2
import requests
from urllib.parse import urlencode
import torch

from data.neuro.letters_in_image import Letters_In_Image
from data.result.Class_text import Class_text
from data.result.Label_area import Label_area
from easyocr import easyocr


class Vagon_number_recognize_yolo:
    def __init__(self, model_path, yolo_path):
        """reads yolov5 taught model from yandex-disk and includes it in class example"""
        base_url = 'https://cloud-api.yandex.net/v1/disk/public/resources/download?'
        public_key = model_path  # Сюда вписываете вашу ссылку
        # Получаем загрузочную ссылку
        final_url = base_url + urlencode(dict(public_key=public_key))
        response = requests.get(final_url)
        download_url = response.json()['href']
        # Загружаем файл и сохраняем его
        download_response = requests.get(download_url)
        zip_path = 'text_recognize_yolo.zip'
        # print(download_response.content)
        with open(zip_path, 'wb') as f:
            f.write(download_response.content)

        with zipfile.ZipFile(zip_path, 'r') as zip_ref:
            zip_ref.extractall()
        # weights_file_path = model_name + '_weights.pt'
        self.model = torch.hub.load(yolo_path, 'custom', 'text_recognize_yolo.pt', source='local')

    def work_image_cut(self, number_image_cut, l, number_h, number_w):
        h, w = number_image_cut.shape[:2]
        hash_object = hashlib.md5(bytes(str(datetime.now()), 'utf-8'))
        hash_str = hash_object.hexdigest()
        img_cut_path = 'cut_text_img' + hash_str + '.jpg'
        cv2.imwrite(img_cut_path, number_image_cut)
        # self.model.max_det = l
        results = self.model(img_cut_path, size=(number_h, number_w))
        json_res = results.pandas().xyxy[0].to_json(orient="records")
        res2 = json.loads(json_res)
        img_letters = Letters_In_Image.get_letters_in_image_from_yolo_json(res2)
        img_letters.sort_letters()
        img_letters.delete_intersections()
        img_letters.delete_x_intersections()
        if len(img_letters.letters) > 8:
            img_letters.letters = img_letters.letters[0:8]
        os.remove(img_cut_path)
        return img_letters.make_word(), img_letters.get_avg_conf()

    def make_cuts(self, img_vagon, rect_array):
        number_image_cuts = []
        for rect in rect_array:
            number_image_cuts.append(img_vagon[rect.ymin:rect.ymax, rect.xmin:rect.xmax])
        return number_image_cuts

    def work_image(self, img_vagon, rect_array):
        image_text_areas = self.make_cuts(img_vagon, rect_array)
        numbers = []
        for image_text_area in image_text_areas:
            number = {'image_text_area' : image_text_area, 'text' : '', 's' : 0, 'conf' : 0.0}
            h, w = image_text_area.shape[:2]
            number['s'] = h * w
            (text, conf) = self.work_image_cut(image_text_area, 8, 128, 196)
            number['text'] = text
            number['conf'] = conf
            numbers.append(number)
        numbers.sort(key= lambda number : number['s'], reverse=True)
        if len(numbers) == 1:
            return (numbers[0]['text'], numbers[0]['conf'])
        else:
            if (len(numbers[0]['text']) != 8) and (len(numbers[1]['text']) == 8):
                return (numbers[1]['text'], numbers[1]['conf'])
            elif (len(numbers[0]['text']) == 8) and (len(numbers[1]['text']) != 8):
                return (numbers[0]['text'], numbers[0]['conf'])
            else:
                ans = sorted(numbers[0:2], key = lambda number : number['conf'], reverse=True)[0]
                return ans['text'], ans['conf']


"""module for rectangle operations"""

class Rect:
    """rectangle class contains xmin, ymin - upperleft and xmax ymax - downright corners of rectangle"""
    # прочитать из json результата йоло
    @staticmethod
    def get_rect_from_yolo_json(yolo_json):
        """from yolo JSON answer reads Rect object"""
        if len(yolo_json) > 0:
            xmin = int(float(yolo_json[0]['xmin']))
            xmax = int(float(yolo_json[0]['xmax']))
            ymin = int(float(yolo_json[0]['ymin']))
            ymax = int(float(yolo_json[0]['ymax']))
            rect = Rect(xmin, xmax, ymin, ymax)
            return rect
        else:
            return None

    def __init__(self, xmin=0, xmax=0, ymin=0, ymax=0):
        """makes Rect object from xmin, xmax, ymin, ymax"""
        self.xmin = xmin
        self.xmax = xmax
        self.ymin = ymin
        self.ymax = ymax

    # Найти IOU между этим прямоугольником и другим, данным в объекте
    def IoU(self, rect):
        """find intersection over union between object and other rectangle"""
        def IOU(xmin, xmax, ymin, ymax, xmin_t, xmax_t, ymin_t, ymax_t,h,w):
            I = 0
            U = 0
            for i in range(0, w):
                for j in range(0, h):
                    flag = ((i <= xmax) and (i >= xmin) and (j <= ymax) and (j >= ymin))
                    flag_t = ((i <= xmax_t) and (i >= xmin_t) and (j <= ymax_t) and (j >= ymin_t))
                    if (flag and flag_t):
                        I += 1
                    if (flag or flag_t):
                        U += 1
            resultat = I / float(U)
            return resultat
        h = max(self.ymax, rect.ymax) - min(self.ymin, rect.ymin)
        w = max(self.xmax, rect.xmax) - min(self.xmin, rect.xmin)
        return IOU(self.xmin, self.xmax, self.ymin, self.ymax,
                   rect.xmin, rect.xmax, rect.ymin, rect.ymax)

    def __str__(self):
        """makes string from object"""
        res = ('xmin = ' + str(self.xmin) + ', xmax = ' + str(self.xmax) + ', ymin = ' + str(self.ymin) +
               ', ymax = ' + str(self.ymax))
        return res

    def intersection(self, rect):
        """find intersection square between object and other rectangle"""
        h = max(self.ymax, rect.ymax) - min(self.ymin, rect.ymin)
        w = max(self.xmax, rect.xmax) - min(self.xmin, rect.xmin)
        I = 0
        U = 0
        for i in range(0, w):
            for j in range(0, h):
                flag = ((i <= self.xmax) and (i >= self.xmin) and (j <= self.ymax) and (j >= self.ymin))
                flag_t = ((i <= rect.xmax) and (i >= rect.xmin) and (j <= rect.ymax) and (j >= rect.ymin))
                if (flag and flag_t):
                    I += 1
        return I

    def union(self, rect):
        """find union RECT between object and other rectangle"""
        new_xmin = min(self.xmin, rect.xmin)
        new_ymin = min(self.ymin, rect.ymin)
        new_xmax = max(self.xmax, rect.xmax)
        new_ymax = max(self.ymax, rect.ymax)
        return Rect(new_xmin, new_xmax, new_ymin, new_ymax)
